@@ -44,16 +44,14 @@ __doc__ = """ExtFile product module.
     The ExtFile-Product works like the Zope File-product, but stores 
     the uploaded file externally in a repository-direcory."""
 
-__version__='$Release: 1.4.4 $'[10:-2]
+__version__='$Release: 1.5.0 $'[10:-2]
 
-import Globals
 from Products.ZCatalog.CatalogPathAwareness import CatalogAware
 from OFS.SimpleItem import SimpleItem
 from OFS.PropertyManager import PropertyManager
-from Globals import HTMLFile, MessageDialog, InitializeClass
+from Globals import HTMLFile, MessageDialog, InitializeClass, package_home
 from AccessControl import ClassSecurityInfo, getSecurityManager
 from AccessControl import Permissions
-from OFS.content_types import guess_content_type
 from mimetypes import guess_extension
 from webdav.Lockable import ResourceLockedError
 from webdav.common import rfc1123_date
@@ -61,7 +59,7 @@ from DateTime import DateTime
 import urllib, os, string, types, sha, base64
 from os.path import join, isfile
 from tempfile import TemporaryFile
-from Products.ExtFile.TM import VTM
+from Products.ExtFile import TM
 
 from webdav.WriteLockInterface import WriteLockInterface
 from IExtFile import IExtFile
@@ -70,6 +68,13 @@ from zLOG import *
 _SUBSYS = 'ExtFile'
 _debug = 0
 
+try: import Zope2
+except ImportError: ZOPE28 = 0
+else: ZOPE28 = 1
+
+try: from zope.app.content_types import guess_content_type
+except ImportError: from OFS.content_types import guess_content_type
+
 try: from zExceptions import Redirect
 except ImportError: Redirect = 'Redirect'
 
@@ -81,11 +86,11 @@ DownloadPermission = 'Download ExtFile/ExtImage'
 import re
 copy_of_re = re.compile('(^(copy[0-9]*_of_)+)')
 
-
 from Config import *
 
-bad_chars =  ' ,;()[]{}ݟ'
-good_chars = '_________AAAAAAaaaaaaCcEEEEEeeeeeIIIIiiiiNnOOOOOOooooooSssUUUUuuuuYYyyZz'
+# FIXME: This is latin-1 only
+bad_chars =  """ ,;:'"()[]{}ݟ"""
+good_chars = """____________AAAAAAaaaaaaCcEEEEEeeeeeIIIIiiiiNnOOOOOOooooooSssUUUUuuuuYYyyZz"""
 TRANSMAP = string.maketrans(bad_chars, good_chars)
 
 manage_addExtFileForm = HTMLFile('dtml/extFileAdd', globals()) 
@@ -94,7 +99,7 @@ manage_addExtFileForm = HTMLFile('dtml/extFileAdd', globals())
 def manage_addExtFile(self, id='', title='', descr='', file='', 
                       content_type='', permission_check=0, redirect_default_view=0, REQUEST=None):
     """ Add an ExtFile to a folder. """
-    if not id and hasattr(file,'filename'): 
+    if not id and getattr(file, 'filename', None) is not None:
         # generate id from filename and make sure, it has no 'bad' chars
         id = file.filename
         id = id[max(string.rfind(id,'/'), 
@@ -108,17 +113,18 @@ def manage_addExtFile(self, id='', title='', descr='', file='',
         self._getOb(id).manage_file_upload(file, content_type)
     if REQUEST is not None:
         return self.manage_main(self, REQUEST, update_menu=0)
+    return id
 
 
 
-class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
+class ExtFile(CatalogAware, SimpleItem, PropertyManager):
     """ The ExtFile-Product works like the Zope File-product, but stores 
         the uploaded file externally in a repository-direcory. """
 
     __implements__ = (IExtFile, WriteLockInterface)
     
     # what properties have we?
-    _properties=(
+    _properties = (
         {'id':'title',                          'type':'string',    'mode': 'w'},
         {'id':'descr',                          'type':'text',      'mode': 'w'},
         {'id':'content_type',                   'type':'string',    'mode': 'w'},
@@ -129,12 +135,12 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     redirect_default_view = 0
     
     # what management options are there?
-    manage_options = (
+    manage_options = ((
         {'label':'Edit',            'action': 'manage_main'             },
-        {'label':'View/Download',   'action': ''                        },
-        {'label':'Upload',          'action': 'manage_uploadForm'       },
-        {'label':'Properties',      'action': 'manage_propertiesForm'   },
-        {'label':'Security',        'action': 'manage_access'           },
+        {'label':'View',            'action': ''                        },
+        {'label':'Upload',          'action': 'manage_uploadForm'       },) +
+        PropertyManager.manage_options +
+        SimpleItem.manage_options[1:]
     ) 
     
     security = ClassSecurityInfo()
@@ -146,8 +152,12 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     _repository = REPOSITORY_PATH
     
     # make sure the download permission is available
-    security.declareProtected(DownloadPermission, 'dummy')
+    security.setPermissionDefault(DownloadPermission, ('Manager',))
     
+    # the above does not work in Zope < 2.8
+    if not ZOPE28:
+        security.declareProtected(DownloadPermission, '_dummy')
+
     # MIME-Type Dictionary. To add a MIME-Type, add a file in the directory 
     # icons/_category_/_subcategory-icon-file_
     # example: Icon tifficon.gif for the MIME-Type image/tiff goes to 
@@ -188,9 +198,11 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     # Public methods               #
     ################################
     
-    def __str__(self): return self.index_html()
+    def __str__(self):
+        return self.index_html()
     
-    def __len__(self): return 1
+    def __len__(self):
+        return 1
     
     security.declareProtected(ViewPermission, 'index_html')
     def index_html (self, icon=0, preview=0, width=None, height=None, 
@@ -201,14 +213,12 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         # Patch provided by Oliver Bleutgen.
         if self.redirect_default_view:
             if self.static_mode() and REQUEST is not None and not icon:
-                static_url = self.static_url(preview=preview)
+                static_url = self._static_url(preview=preview)
                 if static_url != self.absolute_url():
                     return REQUEST.RESPONSE.redirect(static_url)
 
         # HTTP If-Modified-Since header handling. (Copied from OFS/Image.py)
         # XXX: This seems handicapped. How does it handle icon and preview?
-        if REQUEST is None and hasattr(self,'REQUEST'): 
-            REQUEST = self.REQUEST
         if REQUEST is not None:
             header = REQUEST.get_header('If-Modified-Since', None)
             if header is not None:
@@ -222,6 +232,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                         last_mod = long(0)
                     if last_mod > 0 and last_mod < mod_since:
                         # Set headers for Apache caching
+                        last_mod = rfc1123_date(self._p_mtime)
                         REQUEST.RESPONSE.setHeader('Last-Modified', last_mod)
                         REQUEST.RESPONSE.setHeader('Content-Type', self.content_type)
                         # RFC violation. See http://collector.zope.org/Zope/544
@@ -239,7 +250,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             try: size = os.stat(filename)[6]
             except: raise cant_read_exc, ("%s (%s)" %(self.id, filename))
         else:
-            filename = join(Globals.package_home(globals()), 'icons', 'broken.gif')
+            filename = join(package_home(globals()), 'icons', 'broken.gif')
             try: size = os.stat(filename)[6]
             except: raise cant_read_exc, ("%s (%s)" %(self.id, filename))
             content_type = 'image/gif'
@@ -305,10 +316,15 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         """ Return an icon for the file's MIME-Type """
         raise Redirect, self._static_url(icon=1)
     
+    security.declareProtected(ViewPermission, 'icon_tag')
+    def icon_tag(self):
+        """ Generate the HTML IMG tag for the icon """
+        return '<img src="%s" border="0" />' % self._static_url(icon=1)
+    
     security.declareProtected(ViewPermission, 'icon_html')
     def icon_html(self):
-        """ The icon embedded in html with a link to the real file """
-        return '<img src="%s" border="0" />' % self._static_url(icon=1)
+        """ Same as icon_tag """
+        return self.icon_tag()
     
     security.declareProtected(ViewPermission, 'is_broken')
     def is_broken(self):
@@ -323,10 +339,16 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             return os.stat(fn)[6]
         return 0
     
-    # b/w compat
-    rawsize = get_size
-    getSize = get_size
-    
+    security.declareProtected(ViewPermission, 'rawsize')
+    def rawsize(self):
+        """ Same as get_size """
+        return self.get_size()
+
+    security.declareProtected(ViewPermission, 'getSize')
+    def getSize(self):
+        """ Same as get_size """
+        return self.get_size()
+
     security.declareProtected(ViewPermission, 'size')
     def size(self):
         """ Returns a formatted stringified version of the file size """
@@ -345,7 +367,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             directory shold be used as icon for this file/image
         """
         cat, sub = string.split(self.content_type, '/')
-        if hasattr(self,'has_preview') and self.has_preview: cat = 'image'
+        if getattr(self, 'has_preview', 0):
+            cat = 'image'
         if self._types.has_key(cat):
             file = self._types[cat]['default']
             for item in self._types[cat].keys():
@@ -373,6 +396,14 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         """
         return self._fsname(self.filename)
 
+    security.declareProtected(ViewPermission, 'PrincipiaSearchSource')
+    def PrincipiaSearchSource(self):
+        """ Allow file objects to be searched.
+        """
+        if self.content_type.startswith('text/'):
+            return str(self)
+        return ''
+
     ################################
     # Protected management methods #
     ################################
@@ -405,6 +436,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             temp_file = TemporaryFile()
             temp_file.write(file)
             temp_file.seek(0)
+        else:
+            temp_file = file
         return self.manage_file_upload(temp_file, content_type, REQUEST)
 
     security.declareProtected(ChangePermission, 'manage_file_upload')
@@ -426,7 +459,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         try:
             new_fn = self._get_ufn(self.filename)
             self._update_data(file, self._temp_fsname(new_fn))
-        finally: self._dir__unlock()
+        finally:
+            self._dir__unlock()
         self.filename = new_fn
         self._afterUpdate()
         if REQUEST is not None:
@@ -450,7 +484,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         try:
             new_fn = self._get_ufn(self.filename)
             self._update_data(file, self._temp_fsname(new_fn))
-        finally: self._dir__unlock()
+        finally:
+            self._dir__unlock()
         self.filename = new_fn
         self._afterUpdate()
         if REQUEST is not None:
@@ -480,7 +515,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                 path = None
             new_fn = self._get_ufn(self.filename, path=path)
             self._update_data(file, self._temp_fsname(new_fn))
-        finally: self._dir__unlock()
+        finally:
+            self._dir__unlock()
         self.filename = new_fn
         self._afterUpdate()
         RESPONSE.setStatus(204)
@@ -489,16 +525,19 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     security.declareProtected('FTP access', 'manage_FTPstat')
     security.declareProtected('FTP access', 'manage_FTPlist')
     security.declareProtected('FTP access', 'manage_FTPget')
-    manage_FTPget = index_html
+    def manage_FTPget(self):
+        """ Return body for FTP """
+        return self.index_html(REQUEST=self.REQUEST)
     
     ################################
     # Private methods              #
     ################################
 
-    def _access_permitted(self, REQUEST):
+    def _access_permitted(self, REQUEST=None):
         """ Check if the user is allowed to download the file """
-        if hasattr(self,'use_download_permission_check') and \
-           self.use_download_permission_check and \
+        if REQUEST is None and getattr(self, 'REQUEST', None) is not None:
+            REQUEST = self.REQUEST
+        if getattr(self, 'use_download_permission_check', 0) and \
            (REQUEST is None or 
             not getSecurityManager().getUser().has_permission(
                                         DownloadPermission, self)
@@ -516,6 +555,9 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             if type(body) is not type(''): body = body.data
             content_type, enc = guess_content_type(getattr(file,'filename',id),
                                                    body, content_type)
+        cutoff = content_type.find(';')
+        if cutoff >= 0:
+            return content_type[:cutoff]
         return content_type
     
     def _update_data(self, infile, outfile):
@@ -613,15 +655,17 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
         if isfile(tmp_fn):
             return tmp_fn
 
-    # b/w compat
-    _get_filename = _get_fsname
+    # b/w compatibility
+    def _get_filename(self, filename):
+        """ Deprecated, use _get_fsname """
+        return self._get_fsname(filename)
 
     def _get_ufn(self, filename, path=None, content_type=None, lock=1):
         """ If no unique filename has been generated, generate one
             otherwise, return the existing one.
         """
         if UNDO_POLICY==ALWAYS_BACKUP or filename==[]: 
-            new_fn = self._get_new_ufn(path=path, content_type=content_type)
+            new_fn = self._get_new_ufn(path=path, content_type=content_type, lock=lock)
         else: 
             new_fn = filename[:]
         if filename:
@@ -634,7 +678,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                 except OSError: pass
         return new_fn
 
-    def _get_new_ufn(self, path=None, content_type=None, lock=1, copy_of_re=copy_of_re):
+    def _get_new_ufn(self, path=None, content_type=None, lock=1):
         """ Create a new unique filename """
         id = self.id
 
@@ -653,16 +697,23 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             id_name = id
             id_ext = ''
 
+        if not content_type:
+            content_type = self.content_type
+
         if REPOSITORY_EXTENSIONS in (MIMETYPE_APPEND, MIMETYPE_REPLACE):
-            mime_ext = guess_extension(content_type or self.content_type)
+            mime_ext = guess_extension(content_type)
             if mime_ext is not None:
                 if mime_ext in ('.jpeg', '.jpe'): 
                     mime_ext = '.jpg'   # for IE/Win :-(
                 if mime_ext in ('.obj',):
                     mime_ext = '.exe'   # b/w compatibility
-                if REPOSITORY_EXTENSIONS == MIMETYPE_APPEND:
-                    id_name = id_name + id_ext
-                id_ext = mime_ext
+                if mime_ext in ('.tiff',):
+                    mime_ext = '.tif'   # b/w compatibility
+                # don't change extensions of unknown binaries
+                if not (content_type == 'application/octet-stream' and id_ext):
+                    if REPOSITORY_EXTENSIONS == MIMETYPE_APPEND:
+                        id_name = id_name + id_ext
+                    id_ext = mime_ext
         
         # generate directory structure
         if path is not None:
@@ -675,7 +726,9 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             dirs = rel_url_list
         elif REPOSITORY in (SLICED, SLICED_REVERSE, SLICED_HASH):
             if REPOSITORY == SLICED_HASH:
-                temp = base64.encodestring(sha.new(id_name).digest())[:-1]
+                # increase distribution by including the path in the hash
+                hashed = ''.join(list(rel_url_list)+[id_name])
+                temp = base64.encodestring(sha.new(hashed).digest())[:-1]
                 temp = temp.replace('/', '_')
                 temp = temp.replace('+', '_')
             elif REPOSITORY == SLICED_REVERSE: 
@@ -716,7 +769,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             raise invalid_format_exc, FILE_FORMAT
         # user (%u)
         if string.find(fileformat, "%u")>=0:
-            if (hasattr(self, "REQUEST") and 
+            if (getattr(self, 'REQUEST', None) is not None and
                self.REQUEST.has_key('AUTHENTICATED_USER')):
                 user = getSecurityManager().getUser().getUserName()
                 fileformat = string.replace(fileformat, "%u", user)
@@ -724,7 +777,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                 fileformat = string.replace(fileformat, "%u", "")
         # path (%p)
         if string.find(fileformat, "%p")>=0:
-            temp = string.joinfields (rel_url_list, "_")
+            temp = string.joinfields(rel_url_list, "_")
             fileformat = string.replace(fileformat, "%p", temp)
         # file and extension (%n and %e)
         if string.find(fileformat,"%n")>=0 or string.find(fileformat,"%e")>=0:
@@ -736,14 +789,14 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
 
         # search for unique filename
         if counter: 
-            fn = join(dirpath, string.replace(fileformat, "%c", `counter`))
+            fn = join(dirpath, string.replace(fileformat, "%c", ".%s" % counter))
         else: 
             fn = join(dirpath, string.replace(fileformat, "%c", ''))
         while isfile(fn) or isfile(fn+'.undo') or isfile(fn+'.tmp'):
             counter = counter + 1
-            fn = join(dirpath, string.replace(fileformat, "%c", `counter`))
+            fn = join(dirpath, string.replace(fileformat, "%c", ".%s" % counter))
         if counter: 
-            fileformat = string.replace(fileformat, "%c", `counter`)
+            fileformat = string.replace(fileformat, "%c", ".%s" % counter)
         else: 
             fileformat = string.replace(fileformat, "%c", '')
 
@@ -780,17 +833,13 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
 
     def _get_file_to_serve(self, icon=0, preview=0):
         """ Find out about the file we are going to serve """
-        if hasattr(self, 'has_preview') and self.has_preview: 
-            has_preview = 1
-        else: 
-            has_preview = 0
-        if not self._access_permitted(self.REQUEST): 
+        if not self._access_permitted(): 
             preview = 1
-        if preview and not has_preview: 
+        if preview and not getattr(self, 'has_preview', 0): 
             icon = 1
         
         if icon:
-            filename = join(Globals.package_home(globals()), self.getIconPath())
+            filename = join(package_home(globals()), self.getIconPath())
             content_type = 'image/gif'
         elif preview:
             filename = self.prev_filename
@@ -826,7 +875,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
             path = list(parent.getPhysicalPath())
         return filter(None, path)
 
-    def _bytetostring (self, value):
+    def _bytetostring(self, value):
         """ Convert an int-value (file-size in bytes) to an String
             with the file-size in Byte, KB or MB
         """
@@ -877,7 +926,8 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                     self._update_data(fn, self._temp_fsname(new_fn))
                     self.filename = new_fn
                     call_afterUpdate = 1
-                finally: self._dir__unlock()
+                finally:
+                    self._dir__unlock()
                 if call_afterUpdate:
                     self._afterUpdate()
         return ExtFile.inheritedAttribute("manage_afterClone")(self, item)
@@ -955,9 +1005,9 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     ################################
 
     def _register(self):
-        if _debug: LOG(_SUBSYS, INFO, 'registering %s' % self._v_registered)
-        ExtFile.inheritedAttribute('_register')(self)
-        if _debug: LOG(_SUBSYS, INFO, 'registered %s' % self._v_registered)
+        if _debug: LOG(_SUBSYS, INFO, 'registering %s' % TM.contains(self))
+        TM.register(self)
+        if _debug: LOG(_SUBSYS, INFO, 'registered %s' % TM.contains(self))
 
     def _begin(self):
         self._v_begin_called = 1    # for tests
@@ -966,6 +1016,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     def _finish(self):
         """ Commits the temporary file """
         self._v_finish_called = 1   # for tests
+        TM.remove(self)             # for tests
         if self.filename:
             tmp_fn = self._temp_fsname(self.filename)
             if _debug: LOG(_SUBSYS, INFO, 'finishing %s' % tmp_fn) 
@@ -979,6 +1030,7 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
     def _abort(self):
         """ Deletes the temporary file """
         self._v_abort_called = 1    # for tests
+        TM.remove(self)             # for tests
         if self.filename:
             tmp_fn = self._temp_fsname(self.filename)
             if _debug: LOG(_SUBSYS, INFO, 'aborting %s' % tmp_fn) 
@@ -987,26 +1039,30 @@ class ExtFile(CatalogAware, SimpleItem, PropertyManager, VTM):
                 try: os.remove(tmp_fn)
                 except OSError: pass
 
-
 InitializeClass(ExtFile)
 
 
-
+# FileUpload factory
 from cgi import FieldStorage
 from ZPublisher.HTTPRequest import FileUpload
 
-class HTTPUpload(FileUpload):
+def HTTPUpload(fp, content_type=None):
     """ Create a FileUpload instance from a file handle (and content_type) """
-
-    def __init__(self, fp, content_type=None):
+    if isinstance(fp, FileUpload):
+        if content_type:
+            fp.headers['content-type'] = content_type
+        return fp
+    else:
         environ = {'REQUEST_METHOD': 'POST'}
         if content_type:
             environ['CONTENT_TYPE'] = content_type
         elif hasattr(fp, 'headers') and fp.headers.has_key('content-type'):
             environ['CONTENT_TYPE'] = fp.headers['content-type']
-        FileUpload.__init__(self, FieldStorage(fp=fp, environ=environ))
+        fs = FieldStorage(fp=fp, environ=environ)
+        return FileUpload(fs)
 
 
+# Repository lock
 import time
 
 class DirLockError(OSError): 
