@@ -44,7 +44,7 @@ __doc__ = """ExtImage product module.
     uploaded image externally in a repository-direcory. It creates a preview
     of the image (requires PIL)."""
 
-__version__='$Release: 1.4.4 $'[10:-2]
+__version__='$Release: 1.5.0 $'[10:-2]
 
 import Globals
 from Products.ExtFile.ExtFile import *
@@ -79,7 +79,7 @@ def manage_addExtImage(self, id='', title='', descr='', file='', preview='',
                        content_type='', create_prev=0, maxx='', maxy='', 
                        ratio=0, permission_check=0, redirect_default_view=0, REQUEST=None):
     """ Add an ExtImage to a folder. """
-    if not id and hasattr(file,'filename'): 
+    if not id and getattr(file, 'filename', None) is not None:
         # generate id from filename and make sure, it has no 'bad' chars
         id = file.filename
         id = id[max(string.rfind(id,'/'), 
@@ -95,6 +95,7 @@ def manage_addExtImage(self, id='', title='', descr='', file='', preview='',
         self._getOb(id).manage_file_upload(preview, content_type, 1, create_prev, maxx, maxy, ratio)
     if REQUEST is not None:
         return self.manage_main(self, REQUEST, update_menu=0)
+    return id
 
 
 
@@ -144,15 +145,18 @@ class ExtImage(ExtFile):
             the output height and width values of the image tag.
             Adopted and adapted from OFS/Image.py
         """
-        if REQUEST is None and hasattr(self,'REQUEST'): 
-            REQUEST = self.REQUEST
-        if not self._access_permitted(REQUEST): preview = 1
-        if preview or not self.is_webviewable(): 
-            url = self._static_url(preview=1)
-            img_width, img_height = self._getImageSize(self.prev_filename)
-        elif icon:
+        if not self.is_webviewable():
+            preview = 1
+        if not self._access_permitted():
+            preview = 1
+        if preview and not self.has_preview:
+            icon = 1
+        if icon:
             url = self._static_url(icon=1)
             img_width, img_height = (32, 32)
+        elif preview:
+            url = self._static_url(preview=1)
+            img_width, img_height = self._getImageSize(self.prev_filename)
         else:
             url = self._static_url()
             img_width, img_height = self._getImageSize(self.filename)
@@ -183,10 +187,15 @@ class ExtImage(ExtFile):
         """ Return a preview of the image """
         raise Redirect, self._static_url(preview=1)
     
+    security.declareProtected(ViewPermission, 'preview_tag')
+    def preview_tag(self):
+        """ Generates the HTML IMG tag for the preview image """
+        return self.tag(preview=1)
+    
     security.declareProtected(ViewPermission, 'preview_html')
     def preview_html(self):
-        """ The preview-image embedded in html with a link to the real file """
-        return self.tag(preview=1)
+        """ Same as preview_tag """
+        return self.preview_tag()
     
     security.declareProtected(ViewPermission, 'is_broken')
     def is_broken(self):
@@ -205,7 +214,7 @@ class ExtImage(ExtFile):
         else:
             return 0
     
-    security.declareProtected(ViewPermission, 'prev_rawsize')
+    security.declareProtected(ViewPermission, 'get_prev_size')
     def get_prev_size(self):
         """ Returns the size of the preview file """
         fn = self._get_fsname(self.prev_filename)
@@ -213,8 +222,10 @@ class ExtImage(ExtFile):
             return os.stat(fn)[6]
         return 0
 
-    # b/w compat
-    prev_rawsize = get_prev_size
+    security.declareProtected(ViewPermission, 'prev_rawsize')
+    def prev_rawsize(self):
+        """ Same as get_prev_size """
+        return self.get_prev_size()
     
     security.declareProtected(ViewPermission, 'prev_size')
     def prev_size(self):
@@ -243,10 +254,11 @@ class ExtImage(ExtFile):
         
     security.declareProtected(ViewPermission, 'format')
     def format(self):
-        """ Get the Fileformat of the image """
+        """ Get the PIL file format of the image """
+        filename = self._get_fsname(self.filename)
         try:
             from PIL import Image
-            im = Image.open(self._get_fsname(self.filename)) 
+            im = Image.open(filename)
             return im.format
         except:
             return 'unknown'
@@ -293,7 +305,8 @@ class ExtImage(ExtFile):
             try:
                 new_fn = self._get_ufn(self.prev_filename, content_type='image/jpeg')
                 self._createPreview(self.filename, new_fn, maxx, maxy, ratio)
-            finally: self._dir__unlock()
+            finally:
+                self._dir__unlock()
         if REQUEST is None:
             return self.has_preview
         else:
@@ -326,6 +339,8 @@ class ExtImage(ExtFile):
             temp_file = TemporaryFile()
             temp_file.write(file)
             temp_file.seek(0)
+        else:
+            temp_file = file
         return self.manage_file_upload(temp_file, content_type, is_preview,
                                        create_prev, maxx, maxy, ratio, REQUEST)
 
@@ -348,7 +363,8 @@ class ExtImage(ExtFile):
                 try:
                     new_fn = self._get_ufn(self.prev_filename, content_type='image/jpeg')
                     self._update_data(file, self._temp_fsname(new_fn))
-                finally: self._dir__unlock()
+                finally:
+                    self._dir__unlock()
                 self._createPreview(new_fn, new_fn, maxx, maxy, ratio)
             else:
                 if content_type:
@@ -360,7 +376,8 @@ class ExtImage(ExtFile):
                 try:
                     new_fn = self._get_ufn(self.prev_filename, content_type=self.prev_content_type)
                     self._update_data(file, self._temp_fsname(new_fn))
-                finally: self._dir__unlock()
+                finally:
+                    self._dir__unlock()
                 self.prev_filename = new_fn
                 self._initPreview()
         else:
@@ -372,7 +389,8 @@ class ExtImage(ExtFile):
                     try:
                         new_fn = self._get_ufn(self.prev_filename, content_type='image/jpeg')
                         self._createPreview(self.filename, new_fn, maxx, maxy, ratio)
-                    finally: self._dir__unlock()
+                    finally:
+                        self._dir__unlock()
         if REQUEST is not None:
             return self.manage_main(self, REQUEST, manage_tabs_message='Upload complete.')
     
@@ -395,7 +413,8 @@ class ExtImage(ExtFile):
             try:
                 new_fn = self._get_ufn(self.prev_filename, content_type=self.prev_content_type)
                 self._update_data(file, self._temp_fsname(new_fn))
-            finally: self._dir__unlock()
+            finally:
+                self._dir__unlock()
             self.prev_filename = new_fn
             self._initPreview()
         else:
@@ -406,7 +425,8 @@ class ExtImage(ExtFile):
                 try:
                     new_fn = self._get_ufn(self.prev_filename, content_type='image/jpeg')
                     self._createPreview(self.filename, new_fn, width, height, self.prev_ratio)
-                finally: self._dir__unlock()
+                finally:
+                    self._dir__unlock()
         if REQUEST is not None:
             return self.manage_main(self, REQUEST, manage_tabs_message='Upload complete.')
     
@@ -428,7 +448,8 @@ class ExtImage(ExtFile):
                     path = None
                 new_fn = self._get_ufn(self.prev_filename, content_type='image/jpeg', path=path)
                 self._createPreview(self.filename, new_fn, width, height, self.prev_ratio)
-            finally: self._dir__unlock() 
+            finally:
+                self._dir__unlock() 
         return RESPONSE
     
     ################################
@@ -437,9 +458,10 @@ class ExtImage(ExtFile):
     
     def _getImageSize(self, filename):
         """ Return width, height tuple using PIL """
+        filename = self._get_fsname(filename)
         try:
             from PIL import Image
-            im = Image.open(self._get_fsname(filename)) 
+            im = Image.open(filename)
             return im.size[0], im.size[1]
         except:
             return 0, 0
@@ -491,11 +513,11 @@ class ExtImage(ExtFile):
     def _formatDimensions(self, maxx, maxy):
         """ Make sure, the dimensions are valid int's """
         if type(maxx) is types.StringType:
-            try: maxx = string.atoi(maxx)
-            except: maxx = self._image_size['default']
+            try: maxx = int(maxx)
+            except ValueError: maxx = self._image_size['default']
         if type(maxy) is types.StringType:
-            try: maxy = string.atoi(maxy)
-            except: maxy = self._image_size['default']
+            try: maxy = int(maxy)
+            except ValueError: maxy = self._image_size['default']
         if maxx!=0 and maxy!=0:
             if maxx<self._image_size['min']: maxx = self._image_size['min']
             elif maxx>self._image_size['max']: maxx = self._image_size['max']
