@@ -11,14 +11,14 @@ from Testing import ZopeTestCase
 ZopeTestCase.installProduct('ExtFile')
 
 from Products.ExtFile.tests.ExtFileTestCase import ExtFileTestCase
-from Products.ExtFile.tests.ExtFileTestCase import tifImage
+from Products.ExtFile.tests.ExtFileTestCase import tiffImage
 
 
 class TestCMYKPreview(ExtFileTestCase):
 
     def afterSetUp(self):
         ExtFileTestCase.afterSetUp(self)
-        self.addExtImage(id='mountain.tif', file=tifImage)
+        self.addExtImage(id='mountain.tif', file=tiffImage)
 
     def testSetup(self):
         self.assertEqual(self.image.content_type, 'image/tiff')
@@ -34,7 +34,7 @@ class TestCMYKPreview(ExtFileTestCase):
         self.failUnless(self._exists('mountain.jpg.tmp'))
 
     def testCreatePreviewOnUpload(self):
-        self.image.manage_file_upload(tifImage, content_type='image/tiff', create_prev=1)
+        self.image.manage_file_upload(tiffImage, content_type='image/tiff', create_prev=1)
         self.failUnless(self._exists('mountain.jpg.tmp'))
 
 
