@@ -15,6 +15,7 @@ from Products.ExtFile.tests.ExtFileTestCase import ExtFileTestCase
 from Products.ExtFile.tests.ExtFileTestCase import gifImage, jpegImage, notImage
 from Products.ExtFile.tests.ExtFileTestCase import copymove_perms
 from Products.ExtFile.ExtImage import NO_PREVIEW, GENERATE, UPLOAD_NORESIZE, UPLOAD_RESIZE
+from Products.ExtFile import transaction, TM
 from Acquisition import aq_base
 
 
@@ -22,32 +23,32 @@ class TestTransactions(ExtFileTestCase):
     '''Test ExtFile/ExtImage transaction awareness.'''
 
     def beforeClose(self):
-        get_transaction().commit()  # Commit the cleaned-up fixture
+        transaction.commit()  # Commit the cleaned-up fixture
 
     def testAddFileCommit(self):
         self.addExtFile(id='file', file=gifImage)
-        get_transaction().commit()
+        transaction.commit()
         self.failUnless(self._exists('file.exe'))
 
     def testAddFileAbort(self):
         # Aborting the transaction leaves the repository empty
         self.addExtFile(id='file', file=gifImage)
-        get_transaction().commit(1)   # Wuhuu, force object rollback
-        get_transaction().abort()
+        transaction.savepoint(1)   # Wuhuu, force object rollback
+        transaction.abort()
         self.failIf(hasattr(aq_base(self.folder), 'file'))
         self.failIf(self._exists('file.exe'))
         self.failIf(self._exists('file.exe.tmp'))
 
     def testAddImageCommit(self):
         self.addExtImage(id='image', file=gifImage)
-        get_transaction().commit()
+        transaction.commit()
         self.failUnless(self._exists('image.gif'))
 
     def testAddImageAbort(self):
         # Aborting the transaction leaves the repository empty
         self.addExtImage(id='image', file=gifImage)
-        get_transaction().commit(1)   # Wuhuu, force object rollback
-        get_transaction().abort()
+        transaction.savepoint(1)   # Wuhuu, force object rollback
+        transaction.abort()
         self.failIf(hasattr(aq_base(self.folder), 'image'))
         self.failIf(self._exists('image.gif'))
         self.failIf(self._exists('image.gif.tmp'))
@@ -56,8 +57,8 @@ class TestTransactions(ExtFileTestCase):
         # Aborting the transaction leaves the repository empty
         self.addExtImage(id='image', file=gifImage)
         self.image.manage_create_prev(maxx=10, maxy=10)
-        get_transaction().commit(1)   # Wuhuu, force object rollback
-        get_transaction().abort()
+        transaction.savepoint(1)   # Wuhuu, force object rollback
+        transaction.abort()
         self.failIf(hasattr(aq_base(self.folder), 'image'))
         self.failIf(self._exists('image.gif'))
         self.failIf(self._exists('image.gif.tmp'))
@@ -67,36 +68,36 @@ class TestTransactions(ExtFileTestCase):
     def testAddMoreThanOneFileInSeparateTransactions(self):
         # First file
         self.addExtFile(id='fred', file=notImage)
-        get_transaction().commit()
+        transaction.commit()
         self.failUnless(self._exists('fred.exe'))
         self.failIf(self._exists('fred.exe.tmp'))
         # Second file
         self.addExtFile(id='barney', file=notImage)
-        get_transaction().commit()
+        transaction.commit()
         self.failUnless(self._exists('barney.exe'))
         self.failIf(self._exists('barney.exe.tmp'))
         # Third file
         self.addExtFile(id='betty', file=notImage)
-        get_transaction().commit()
+        transaction.commit()
         self.failUnless(self._exists('betty.exe'))
         self.failIf(self._exists('betty.exe.tmp'))
 
     def testUploadFileIntoExistingInSeparateTransactions(self):
         # Create a file 'fred'
         self.addExtFile(id='fred', file=notImage)
-        self.assertEqual(self.file._v_registered, 1)
+        self.failUnless(TM.contains(self.file))
         self.failUnless(self._exists('fred.exe.tmp'))
-        get_transaction().commit()
-        self.assertEqual(self.file._v_registered, 0)
+        transaction.commit()
+        self.failIf(TM.contains(self.file))
         self.failUnless(self._exists('fred.exe'))
         self.failIf(self._exists('fred.exe.tmp'))
         # Upload new file into 'fred'
         self.file.manage_file_upload(file=gifImage)
-        self.assertEqual(self.file._v_registered, 1)
+        self.failUnless(TM.contains(self.file))
         self.failUnless(self._exists('fred.exe'))
         self.failUnless(self._exists('fred.exe.tmp'))
-        get_transaction().commit()
-        self.assertEqual(self.file._v_registered, 0)
+        transaction.commit()
+        self.failIf(TM.contains(self.file))
         self.failUnless(self._exists('fred.exe'))
         self.failIf(self._exists('fred.exe.tmp'))
 
@@ -109,17 +110,17 @@ class TestTransactionManager(ExtFileTestCase):
         self.file._register()
 
     def beforeClose(self):
-        get_transaction().commit()  # Commit the cleaned-up fixture
+        transaction.commit()  # Commit the cleaned-up fixture
 
     def testBegin(self):
         self.assertEqual(self.file._v_begin_called, 1)
 
     def testFinish(self):
-        get_transaction().commit()
+        transaction.commit()
         self.assertEqual(self.file._v_finish_called, 1)
 
     def testAbort(self):
-        get_transaction().abort()
+        transaction.abort()
         self.assertEqual(self.file._v_abort_called, 1)
 
 
@@ -132,7 +133,7 @@ class TestExtFileTransactions(ExtFileTestCase):
         self.subfolder = self.folder.subfolder
 
     def beforeClose(self):
-        get_transaction().commit()  # Commit the cleaned-up fixture
+        transaction.commit()  # Commit the cleaned-up fixture
 
     def testManageFileUploadCreatesTempFile(self):
         self.addExtFile(id='file', file='')
@@ -142,7 +143,7 @@ class TestExtFileTransactions(ExtFileTestCase):
 
     def testManageHTTPUploadCreatesTempFile(self):
         self.addExtFile(id='file', file='')
-        self.file.manage_http_upload(url=self.app.ZopeFile.absolute_url())
+        self.file.manage_http_upload(url=self.app.NotImage.absolute_url())
         self.failUnless(self._exists('file.exe.tmp'), 'Missing .tmp file')
         self.failIf(self._exists('file.exe'), 'No .tmp file used')
 
@@ -240,7 +241,7 @@ class TestExtFileTransactions(ExtFileTestCase):
     def testManageHTTPUploadRegistersWithTM(self):
         self.addExtFile(id='file', file='')
         self.assertEqual(getattr(self.file, '_v_begin_called', 0), 0)
-        self.file.manage_http_upload(url=self.app.ZopeFile.absolute_url())
+        self.file.manage_http_upload(url=self.app.NotImage.absolute_url())
         self.assertEqual(getattr(self.file, '_v_begin_called', 0), 1)
 
     def testPUTRegistersWithTM(self):
@@ -255,7 +256,7 @@ class TestExtFileTransactions(ExtFileTestCase):
         self.addExtFile(id='file', file=notImage)
         os.rename(self._fsname('file.exe.tmp'), self._fsname('file.exe.undo'))
         self.file._v_begin_called = 0   # Clear
-        self.file._v_registered = 0     # Clear
+        TM.remove(self.file)            # Clear
         self.assertEqual(getattr(self.file, '_v_begin_called', 0), 0)
         self.file._undo()
         self.assertEqual(getattr(self.file, '_v_begin_called', 0), 1)
@@ -265,7 +266,7 @@ class TestExtFileTransactions(ExtFileTestCase):
         self.addExtFile(id='file', file=notImage)
         self.failUnless(self._exists('file.exe.tmp'))
         fn = self.file._get_new_ufn(self.file.filename)
-        self.assertEqual(fn, ['file1.exe'])
+        self.assertEqual(fn, ['file.1.exe'])
 
     def testGetFileToServeDoesNotUseTempFile(self):
         self.addExtFile(id='file', file=notImage)
@@ -302,43 +303,43 @@ class TestExtFileTransactions(ExtFileTestCase):
     def testManageAfterCloneCreatesTempFile(self):
         self.addExtFile(id='file', file=notImage)
         self.file._finish()
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         cb = self.folder.manage_copyObjects(['file'])
         self.subfolder.manage_pasteObjects(cb)
         self.failUnless(self._exists('file.exe'))       # original
-        self.failUnless(self._exists('file1.exe.tmp'))  # copy
-        self.assertEqual(self.subfolder.file.filename, ['file1.exe'])
+        self.failUnless(self._exists('file.1.exe.tmp'))  # copy
+        self.assertEqual(self.subfolder.file.filename, ['file.1.exe'])
 
     def testManageAfterCloneUsesTempFileAsSource(self):
         self.addExtFile(id='file', file=notImage)
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.file, 'file')
         self.failUnless(self._exists('file.exe.tmp'))   # original
-        self.failUnless(self._exists('file1.exe.tmp'))  # copy
+        self.failUnless(self._exists('file.1.exe.tmp'))  # copy
 
     def testManageAfterCloneUsesMainFileIfTempFileNotPresent(self):
         self.addExtFile(id='file', file=notImage)
         self.file._finish()
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.file, 'file')
         self.failUnless(self._exists('file.exe'))       # original
-        self.failUnless(self._exists('file1.exe.tmp'))  # copy
+        self.failUnless(self._exists('file.1.exe.tmp'))  # copy
 
     def testManageAfterCloneTriesToUndoIfTempFileNotPresent(self):
         self.addExtFile(id='file', file=notImage)
         self.file._finish()
         os.rename(self._fsname('file.exe'), self._fsname('file.exe.undo'))
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.file, 'file')
         self.failUnless(self._exists('file.exe.tmp'))   # restored original
-        self.failUnless(self._exists('file1.exe.tmp'))  # copy
+        self.failUnless(self._exists('file.1.exe.tmp'))  # copy
 
     def testManageAfterCloneRegistersWithTM(self):
         self.addExtFile(id='file', file=notImage)
         self.file._finish()
         self.file._v_begin_called = 0   # Clear
-        self.file._v_registered = 0     # Clear
-        get_transaction().commit(1) # Need a _p_oid
+        TM.remove(self.file)            # Clear
+        transaction.savepoint(1) # Need a _p_oid
         self.assertEqual(getattr(self.subfolder.file, '_v_begin_called', 0), 0)
         self.subfolder.manage_clone(self.file, 'file')
         self.assertEqual(getattr(self.subfolder.file, '_v_begin_called', 0), 1)
@@ -353,7 +354,7 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.subfolder = self.folder.subfolder
 
     def beforeClose(self):
-        get_transaction().commit()  # Commit the cleaned-up fixture
+        transaction.commit()  # Commit the cleaned-up fixture
 
     def testManageFileUploadCreatesTempImage(self):
         self.addExtImage(id='image', file='')
@@ -378,18 +379,18 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.image.manage_file_upload(file=jpegImage, create_prev=GENERATE, maxx=10, maxy=10)
         self.failUnless(self._exists('image.jpg.tmp'), 'Missing .tmp file') # main image
         self.failIf(self._exists('image.jpg'), 'No .tmp file used')
-        self.failUnless(self._exists('image1.jpg.tmp'), 'Missing .tmp file') # preview
-        self.failIf(self._exists('image1.jpg'), 'No .tmp file used')
+        self.failUnless(self._exists('image.1.jpg.tmp'), 'Missing .tmp file') # preview
+        self.failIf(self._exists('image.1.jpg'), 'No .tmp file used')
 
     def testManageHTTPUploadCreatesTempImage(self):
         self.addExtImage(id='image', file='')
-        self.image.manage_http_upload(url=self.app.ZopeImage.absolute_url())
+        self.image.manage_http_upload(url=self.app.GifImage.absolute_url())
         self.failUnless(self._exists('image.gif.tmp'), 'Missing .tmp file')
         self.failIf(self._exists('image.gif'), 'No .tmp file used')
 
     def testManageHTTPUploadCreatesTempPreview(self):
         self.addExtImage(id='image', file='')
-        self.image.manage_http_upload(url=self.app.ZopeImage.absolute_url(), is_preview=1)
+        self.image.manage_http_upload(url=self.app.GifImage.absolute_url(), is_preview=1)
         self.failUnless(self._exists('image.gif.tmp'), 'Missing .tmp file')
         self.failIf(self._exists('image.gif'), 'No .tmp file used')
 
@@ -397,7 +398,7 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         preview_size = self.image.get_prev_size()
-        self.image.manage_http_upload(url=self.app.ZopeImage.absolute_url())
+        self.image.manage_http_upload(url=self.app.GifImage.absolute_url())
         self.failUnless(self._exists('image.gif.tmp'), 'Missing .tmp file') # main image
         self.failIf(self._exists('image.gif'), 'No .tmp file used')
         self.failUnless(self._exists('image.jpg.tmp'), 'Missing .tmp file') # preview
@@ -438,12 +439,12 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file=jpegImage)
         self.image.manage_create_prev(maxx=10, maxy=10)
         self.failUnless(self._exists('image.jpg.tmp'))
-        self.failUnless(self._exists('image1.jpg.tmp'))
+        self.failUnless(self._exists('image.1.jpg.tmp'))
         self.image._finish()
         self.failUnless(self._exists('image.jpg'))
         self.failIf(self._exists('image.jpg.tmp'))
-        self.failUnless(self._exists('image1.jpg'))
-        self.failIf(self._exists('image1.jpg.tmp'))
+        self.failUnless(self._exists('image.1.jpg'))
+        self.failIf(self._exists('image.1.jpg.tmp'))
 
     def testAbortNukesTempImage(self):
         self.addExtImage(id='image', file=jpegImage)
@@ -456,12 +457,12 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file=jpegImage)
         self.image.manage_create_prev(maxx=10, maxy=10)
         self.failUnless(self._exists('image.jpg.tmp'))
-        self.failUnless(self._exists('image1.jpg.tmp'))
+        self.failUnless(self._exists('image.1.jpg.tmp'))
         self.image._abort()
         self.failIf(self._exists('image.jpg'))
         self.failIf(self._exists('image.jpg.tmp'))
-        self.failIf(self._exists('image1.jpg'))
-        self.failIf(self._exists('image1.jpg.tmp'))
+        self.failIf(self._exists('image.1.jpg'))
+        self.failIf(self._exists('image.1.jpg.tmp'))
 
     def testUndoCreatesTempImage(self):
         self.addExtImage(id='image', file=jpegImage)
@@ -476,17 +477,17 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.image.manage_create_prev(maxx=10, maxy=10)
         self.image._finish()
         os.rename(self._fsname('image.jpg'), self._fsname('image.jpg.undo'))
-        os.rename(self._fsname('image1.jpg'), self._fsname('image1.jpg.undo'))
+        os.rename(self._fsname('image.1.jpg'), self._fsname('image.1.jpg.undo'))
         self.image._undo()
         self.failIf(self._exists('image.jpg.undo'))
         self.failUnless(self._exists('image.jpg.tmp'))
-        self.failIf(self._exists('image1.jpg.undo'))
-        self.failUnless(self._exists('image1.jpg.tmp'))
+        self.failIf(self._exists('image.1.jpg.undo'))
+        self.failUnless(self._exists('image.1.jpg.tmp'))
 
     def testIsBrokenReturnsTrueIfPreviewBroken(self):
         self.addExtImage(id='image', file=jpegImage)
         self.image.manage_create_prev(maxx=10, maxy=10)
-        os.remove(self._fsname('image1.jpg.tmp'))
+        os.remove(self._fsname('image.1.jpg.tmp'))
         self.failUnless(self.image.is_broken())
 
     def testManageFileUploadRegistersPreviewWithTM(self):
@@ -511,25 +512,25 @@ class TestExtImageTransactions(ExtFileTestCase):
     def testManageHTTPUploadRegistersPreviewWithTM(self):
         self.addExtImage(id='image', file='')
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 0)
-        self.image.manage_http_upload(url=self.app.ZopeImage.absolute_url(), is_preview=1)
+        self.image.manage_http_upload(url=self.app.GifImage.absolute_url(), is_preview=1)
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 1)
 
     # XXX: Not testable atm
     def DISABLED_testManageHTTPUploadRegistersPreviewWithTMIfAutogen(self):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
-        self.image._v_registered = 0    # Clear
         self.image._v_begin_called = 0  # Clear
+        TM.remove(self.image)           # Clear
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 0)
-        self.image.manage_http_upload(url=self.app.ZopeImage.absolute_url())
+        self.image.manage_http_upload(url=self.app.GifImage.absolute_url())
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 1)
 
     # XXX: Not testable atm
     def DISABLED_testPUTRegistersPreviewWithTMIfAutogen(self):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
-        self.image._v_registered = 0    # Clear
         self.image._v_begin_called = 0  # Clear
+        TM.remove(self.image)           # Clear
         request = self.app.REQUEST
         request['BODYFILE'] = open(jpegImage, 'rb')
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 0)
@@ -540,8 +541,8 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         os.rename(self._fsname('image.jpg.tmp'), self._fsname('image.jpg.undo'))
-        self.image._v_registered = 0    # Clear
         self.image._v_begin_called = 0  # Clear
+        TM.remove(self.image)           # Clear
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 0)
         self.image._undo()
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 1)
@@ -583,53 +584,53 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         self.image._finish()
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         cb = self.folder.manage_copyObjects(['image'])
         self.subfolder.manage_pasteObjects(cb)
         self.failUnless(self._exists('image.jpg'))      # original
-        self.failUnless(self._exists('image1.jpg.tmp')) # copy
-        self.assertEqual(self.subfolder.image.prev_filename, ['image1.jpg'])
+        self.failUnless(self._exists('image.1.jpg.tmp')) # copy
+        self.assertEqual(self.subfolder.image.prev_filename, ['image.1.jpg'])
 
     def testManageAfterCloneUsesTempPreviewAsSource(self):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.image, 'image')
         self.failUnless(self._exists('image.jpg.tmp'))  # original
-        self.failUnless(self._exists('image1.jpg.tmp')) # copy
+        self.failUnless(self._exists('image.1.jpg.tmp')) # copy
 
     def testManageAfterCloneUsesPreviewIfTempPreviewNotPresent(self):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         self.image._finish()
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.image, 'image')
         self.failUnless(self._exists('image.jpg'))      # original
-        self.failUnless(self._exists('image1.jpg.tmp')) # copy
+        self.failUnless(self._exists('image.1.jpg.tmp')) # copy
 
     def testManageAfterCloneTriesToUndoIfTempPreviewNotPresent(self):
         self.addExtImage(id='image', file='')
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         self.image._finish()
         os.rename(self._fsname('image.jpg'), self._fsname('image.jpg.undo'))
-        get_transaction().commit(1) # Need a _p_oid
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.image, 'image')
         self.failUnless(self._exists('image.jpg.tmp'))  # restored original
-        self.failUnless(self._exists('image1.jpg.tmp')) # copy
+        self.failUnless(self._exists('image.1.jpg.tmp')) # copy
 
     def testManageAfterCloneUsesTempPreviewIfOneFile(self):
         # XXX: Fishy. It seems this tests an impossible state
         self.addExtImage(id='image', file=jpegImage)
         self.image.manage_file_upload(file=jpegImage, is_preview=1)
         # Simulate main and preview being one file
-        self.image.prev_filename = ['image2.jpg']
-        get_transaction().commit(1) # Need a _p_oid
+        self.image.prev_filename = ['image.2.jpg']
+        transaction.savepoint(1) # Need a _p_oid
         self.subfolder.manage_clone(self.image, 'image')
         self.failUnless(self._exists('image.jpg.tmp'))  # original
-        self.failUnless(self._exists('image1.jpg.tmp')) # original preview
-        self.failUnless(self._exists('image2.jpg.tmp')) # copy
-        self.assertEqual(self.subfolder.image.prev_filename, ['image2.jpg'])
-        self.assertEqual(self.subfolder.image.filename, ['image2.jpg'])
+        self.failUnless(self._exists('image.1.jpg.tmp')) # original preview
+        self.failUnless(self._exists('image.2.jpg.tmp')) # copy
+        self.assertEqual(self.subfolder.image.prev_filename, ['image.2.jpg'])
+        self.assertEqual(self.subfolder.image.filename, ['image.2.jpg'])
 
     # XXX: Not testable atm
     def DISABLED_testManageAfterCloneRegistersWithTM(self):
@@ -639,14 +640,14 @@ class TestExtImageTransactions(ExtFileTestCase):
         self.addExtImage(id='image', file=jpegImage)
         self.image.manage_create_prev(maxx=10, maxy=10)
         self.failUnless(self._exists('image.jpg.tmp'))
-        self.failUnless(self._exists('image1.jpg.tmp')) # preview
-        self.assertEqual(self.image.prev_filename, ['image1.jpg'])
+        self.failUnless(self._exists('image.1.jpg.tmp')) # preview
+        self.assertEqual(self.image.prev_filename, ['image.1.jpg'])
 
     def testCreatePrevRegistersWithTM(self):
         self.addExtImage(id='image', file=jpegImage)
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 1)
-        self.image._v_registered = 0    # Clear
         self.image._v_begin_called = 0  # Clear
+        TM.remove(self.image)           # Clear
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 0)
         self.image.manage_create_prev(maxx=10, maxy=10)
         self.assertEqual(getattr(self.image, '_v_begin_called', 0), 1)
